using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Todo.Core.Interfaces.Persistence;

namespace Todo.Api.Controllers;

[ApiController]
[Route("api/auth")]
[Authorize]
[AllowAnonymous]
public class AuthController : ControllerBase
{
    private readonly IUserRepository _userRepository;

    public AuthController(IUserRepository userRepository) => _userRepository = userRepository;

    [HttpPost("register")]
    public async Task<IActionResult> Register(
        [FromBody] RegisterUserRequest request)
    {
        var user = await _userRepository.Register(
            request.Email,
            request.Password);

        return Ok(user);
    }

    [HttpGet("login")]
    public async Task<IActionResult> Login([FromQuery] string returnUrl)
    {
        var props = new AuthenticationProperties
        {
            RedirectUri = Url.Action(nameof(Callback)),
            Items =
            {
                {
                    "returnUrl", returnUrl
                }
            }
        };
        return Challenge(props);
    }

    [HttpGet]
    public async Task<IActionResult> Callback()
    {
        // read external identity from the temporary cookie
        var result =
            await HttpContext.AuthenticateAsync("oidc");
        if (result?.Succeeded != true)
            throw new Exception("External authentication error");

        var returnUrl = result.Properties?.Items["returnUrl"] ?? "~/";
        return Redirect(returnUrl);
    }

    public record RegisterUserRequest
    {
        [Required]
        public string Email { get; init; }

        [Required]
        public string Password { get; init; }
    }
}